import React, {Component} from 'react'
import PropTypes from 'prop-types'
import moment from 'moment'
import { Button, Modal, notification, message } from 'antd'

import asyncSpinComponent from '@/utils/asyncSpinComponent'
import Utils, { getSysDefaultSql } from '@/utils/utils.js'
import options from '@/store/options.js'
import { updateForm } from '@/utils/utils-update.js'
import Api from '@/api'
import './index.scss'

const { confirm } = Modal
const MutilForm = asyncSpinComponent(() => import('@/tabviews/zshare/mutilform'))

class MainAction extends Component {
  static propTpyes = {
    BID: PropTypes.string,            // 主表ID
    BData: PropTypes.any,             // 主表数据
    Tab: PropTypes.any,               // 如果当前元素为标签时，tab为标签信息
    type: PropTypes.string,           // 判断当前为主表（main）、子表（sub）、子表标签（subtab）
    MenuID: PropTypes.string,         // 菜单ID
    actions: PropTypes.array,         // 按钮组
    columns: PropTypes.array,         // 显示列
    dict: PropTypes.object,           // 字典项
    setting: PropTypes.any,           // 页面通用设置
    ContainerId: PropTypes.any,       // tab页面ID，用于弹窗控制
    refreshdata: PropTypes.func,      // 执行完成后数据刷新
    selectedData: PropTypes.array     // 表格选中数据
  }

  state = {
    visible: false,
    formdata: null,
    tabledata: null,
    confirmLoading: false,
    execAction: null,
    loadingUuid: '',
    configMap: {},
    loadingNumber: '',
  }

  refreshdata = (item, type) => {
    this.props.refreshdata(item, type)
  }
  
  /**
   * @description 触发按钮操作
   */
  actionTrigger = (item, record) => {
    const { setting, type, Tab, BID, selectedData } = this.props

    if (type === 'sub' && Tab && Tab.supMenu && !BID) {
      notification.warning({
        top: 92,
        message: '需要上级主键值！',
        duration: 5
      })
      return
    }

    let _this = this
    let data = selectedData || []
    
    if (record) { // 表格中触发按钮
      data = [record]
    }

    if (item.Ot !== 'notRequired' && data.length === 0) {
      // 需要选择行时，校验数据
      notification.warning({
        top: 92,
        message: this.props.dict['main.action.confirm.selectline'],
        duration: 5
      })
      return
    } else if (item.Ot === 'requiredSgl' && data.length !== 1) {
      // 需要选择单行时，校验数据
      notification.warning({
        top: 92,
        message: this.props.dict['main.action.confirm.selectSingleLine'],
        duration: 10
      })
      return
    } else if (item.Ot !== 'notRequired' && !setting.primaryKey) {
      // 需要选择行时，校验是否设置主键
      notification.warning({
        top: 92,
        message: '未设置主键！',
        duration: 5
      })
      return
    }

    if (item.OpenType === 'prompt') {
      confirm({
        title: this.props.dict['main.action.confirm.tip'],
        onOk() {
          _this.setState({
            loadingUuid: item.uuid
          })

          return new Promise(resolve => {
            _this.execSubmit(item, data, resolve)
          })
        },
        onCancel() {}
      })
    } else if (item.OpenType === 'exec') {
      this.setState({loadingUuid: item.uuid})
      this.execSubmit(item, data, () => {
        this.setState({loadingUuid: ''})
      })
    } else if (item.OpenType === 'pop') {
      item = updateForm(item)
      if (item.setting.display === 'prompt') { // 如果表单以是否框展示，不请求下拉菜单信息
        this.setState({
          execAction: item,
          tabledata: data
        }, () => {
          this.modelconfirm()
        })
      } else {
        this.setState({
          visible: true,
          execAction: item,
          tabledata: data
        })
      }
    } else {
      notification.warning({
        top: 92,
        message: '尚不支持此功能。。。',
        duration: 5
      })
    }
  }

  /**
   * @description 按钮提交执行
   */
  execSubmit = (btn, data, _resolve, formdata) => {
    const { setting, columns } = this.props

    if (btn.intertype === 'inner') {
      // 执行方式为多行拼接，且打开方式为表单时，会转为循环发送请求
      // 打开方式为模态框，使用内部函数添加(有批量添加场景，已去除)
      if (
        btn.Ot === 'notRequired' ||
        btn.Ot === 'requiredSgl' ||
        (btn.Ot === 'requiredOnce' && btn.OpenType !== 'pop')
      ) {

        let param = { // 系统存储过程
          func: 'sPC_TableData_InUpDe',
          BID: this.props.BID
        }
        let primaryId = setting.primaryKey && data[0] ? (data[0][setting.primaryKey] || '') : ''

        if (btn.OpenType === 'prompt' || btn.OpenType === 'exec') { // 是否弹框或直接执行
          let ID = ''
          if (btn.Ot === 'notRequired') {
            
          } else if (btn.Ot === 'requiredSgl') {
            ID = data[0][setting.primaryKey]
          } else if (btn.Ot === 'requiredOnce') { // id值拼接
            let ids = data.map(d => { return d[setting.primaryKey]})
            ID = ids.join(',')
          }

          if (btn.innerFunc) { // 使用自定义函数
            param.func = btn.innerFunc
            if (setting.primaryKey) { // 主键存在时，设置主键参数
              param[setting.primaryKey] = ID
            }
          } else if (btn.sql) {
            param.ID = primaryId
            param.LText = getSysDefaultSql(btn, setting, '', param, data[0], columns) // 数据源
            
            param.exec_type = 'y' // 后台解码
            param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
            param.secretkey = Utils.encrypt('', param.timestamp)
            param.LText = Utils.formatOptions(param.LText)
          }
        } else if (btn.OpenType === 'pop') { // 表单
          if (btn.innerFunc) {
            param.func = btn.innerFunc

            if (setting.primaryKey) { // 主键存在时，设置主键参数
              param[setting.primaryKey] = primaryId
            }

            formdata.forEach(_data => {
              param[_data.key] = _data.value
            })

          } else if (btn.sql && btn.sqlType === 'insert') { // 系统函数添加时，生成uuid
            primaryId = ''
            if (formdata && setting.primaryKey) { // 表单中存在主键字段，主键值以表单中的值为准
              let _form = formdata.filter(_form => _form.key === setting.primaryKey)[0]
              if (_form) {
                primaryId = _form.value
              }
            }
            param.ID = primaryId || Utils.getguid()
            param.LText = getSysDefaultSql(btn, setting, formdata, param, data[0], columns) // 数据源
            
            param.exec_type = 'y' // 后台解码
            param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
            param.secretkey = Utils.encrypt('', param.timestamp)
            param.LText = Utils.formatOptions(param.LText)
          } else if (btn.sql) {
            param.ID = primaryId
            param.LText = getSysDefaultSql(btn, setting, formdata, param, data[0], columns) // 数据源
            
            param.exec_type = 'y' // 后台解码
            param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
            param.secretkey = Utils.encrypt('', param.timestamp)
            param.LText = Utils.formatOptions(param.LText)
          }
        }

        if (param.timestamp) { // 云端验证
          param.open_key = Utils.encryptOpenKey(param.secretkey, param.timestamp)
        }

        Api.genericInterface(param).then((res) => {
          if (res.status) {
            this.execSuccess(btn, res)
          } else {
            this.execError(res, btn)
          }
          _resolve()
        })
      } else if (btn.Ot === 'required' || (btn.Ot === 'requiredOnce' && btn.OpenType === 'pop')) {
        let _formPrimaryId = ''
        if (formdata && setting.primaryKey) { // 表单中存在主键字段，主键值以表单中的值为准
          let _form = formdata.filter(_form => _form.key === setting.primaryKey)[0]
          if (_form) {
            _formPrimaryId = _form.value
          }
        }

        let _params = data.map((cell, index) => {
          let param = {
            func: 'sPC_TableData_InUpDe',
            BID: this.props.BID
          }
          let primaryId = setting.primaryKey ? cell[setting.primaryKey] : ''

          if (btn.OpenType === 'prompt' || btn.OpenType === 'exec') { // 是否弹框或直接执行

            if (btn.innerFunc) {
              param.func = btn.innerFunc
              if (setting.primaryKey) {
                param[setting.primaryKey] = primaryId
              }
            } else if (btn.sql) {
              param.ID = primaryId
              param.LText = getSysDefaultSql(btn, setting, '', param, cell, columns) // 数据源

              param.exec_type = 'y' // 后台解码
              param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
              param.secretkey = Utils.encrypt('', param.timestamp)
              param.LText = Utils.formatOptions(param.LText)
            }
          } else if (btn.OpenType === 'pop') { // 表单
            if (btn.innerFunc) {
              param.func = btn.innerFunc

              formdata.forEach(_data => {
                if (index !== 0 && _data.readin && cell.hasOwnProperty(_data.key)) {
                  param[_data.key] = cell[_data.key]
                } else {
                  param[_data.key] = _data.value
                }
              })

              if (setting.primaryKey) {
                param[setting.primaryKey] = primaryId
              }
            } else if (btn.sql && btn.sqlType === 'insert') { // 系统函数添加时，生成uuid
              param.ID = _formPrimaryId || Utils.getguid()
              param.LText = getSysDefaultSql(btn, setting, formdata, param, cell, columns) // 数据源

              param.exec_type = 'y' // 后台解码
              param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
              param.secretkey = Utils.encrypt('', param.timestamp)
              param.LText = Utils.formatOptions(param.LText)
            } else if (btn.sql) {
              if (index !== 0) {
                formdata = formdata.map(_data => {
                  if (_data.readin && cell.hasOwnProperty(_data.key)) {
                    _data.value = cell[_data.key]
                  }
                  return _data
                })
              }

              param.ID = primaryId
              param.LText = getSysDefaultSql(btn, setting, formdata, param, cell, columns) // 数据源

              param.exec_type = 'y' // 后台解码
              param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
              param.secretkey = Utils.encrypt('', param.timestamp)
              param.LText = Utils.formatOptions(param.LText)
            }
          }

          if (param.timestamp) { // 云端验证
            param.open_key = Utils.encryptOpenKey(param.secretkey, param.timestamp)
          }

          return param
        })

        if (_params.length <= 20) {
          let deffers = _params.map(param => {
            return new Promise(resolve => {
              Api.genericInterface(param).then(res => {
                resolve(res)
              })
            })
          })
          Promise.all(deffers).then(result => {
            let iserror = false
            let errorMsg = ''
            result.forEach(res => {
              if (res.status) {
                errorMsg = res
              } else {
                iserror = true
                errorMsg = res
              }
            })
            if (!iserror) {
              this.execSuccess(btn, errorMsg)
            } else {
              this.execError(errorMsg, btn)
            }
            _resolve()
          })
        } else { // 超出20个请求时循环执行
          if (btn.progress === 'progressbar') {
            this.setState({
              loadingTotal: _params.length
            })
          }
          this.innerLoopRequest(_params, btn, _resolve)
        }
      }
    } else if (btn.intertype === 'outer') {
      /** *********************调用外部接口************************* */

      let _params = [] // 请求参数数组

      if (btn.Ot === 'notRequired' || btn.Ot === 'requiredSgl' || btn.Ot === 'requiredOnce') {
        let param = {
          BID: this.props.BID
        }

        if (btn.OpenType === 'pop' && formdata) { // 表单
          formdata.forEach(_data => {
            param[_data.key] = _data.value
          })
        }

        // 获取id
        if (btn.Ot === 'notRequired') {
          
        } else if (btn.Ot === 'requiredSgl' && setting.primaryKey) {
          param[setting.primaryKey] = data[0][setting.primaryKey]
        } else if (btn.Ot === 'requiredOnce' && setting.primaryKey) {
          let ids = data.map(d => { return d[setting.primaryKey]})
          param[setting.primaryKey] = ids.join(',')
        }

        _params.push(param)
      } else if (btn.Ot === 'required') {
        // 选择多行，循环调用
        _params = data.map((cell, index) => {
          let _cell = {
            BID: this.props.BID,
          }

          let _formparam = {}
          if (btn.OpenType === 'pop' && formdata) { // 表单
            formdata.forEach(_data => {
              if (index !== 0 && _data.readin && cell.hasOwnProperty(_data.key)) {
                _formparam[_data.key] = cell[_data.key]
              } else {
                _formparam[_data.key] = _data.value
              }
            })
          }

          if (setting.primaryKey) {
            _cell[setting.primaryKey] = cell[setting.primaryKey]
          }

          _cell = {..._formparam, ..._cell}

          return _cell
        })
      }

      if (_params.length > 1 && btn.progress === 'progressbar') {
        this.setState({
          loadingTotal: _params.length
        })
      }

      // 循环调用外部接口（包括内部及回调函数）
      this.outerLoopRequest(_params, btn, _resolve)
    }
  }

  /**
   * @description 内部请求循环执行
   */
  innerLoopRequest = (params, btn, _resolve) => {
    if (!params && params.length === 0) return

    let param = params.shift()

    this.setState({
      loadingNumber: params.length
    })

    Api.genericInterface(param).then(res => {
      if (res.status) {
        if (params.length === 0) {
          this.execSuccess(btn, res)
          _resolve()
        } else {
          this.innerLoopRequest(params, btn, _resolve)
        }
      } else {
        this.execError(res, btn)
        _resolve()
      }
    })
  }

  /**
   * @description 外部请求循环执行
   */
  outerLoopRequest = (params, btn, _resolve) => {
    if (!params && params.length === 0) return

    let param = params.shift()
    let _outParam = null
    let _localParam = null
    let errRes = null

    this.setState({
      loadingNumber: params.length
    })

    new Promise(resolve => {
      // 内部请求
      if (btn.innerFunc) {
        param.func = btn.innerFunc
        // 存在内部函数时，数据预处理
        Api.genericInterface(param).then(res => {
          if (res.status) {
            delete res.ErrCode
            delete res.ErrMesg
            delete res.message
            delete res.status

            // 使用处理后的数据调用外部接口
            let keys = Object.keys(res) // 提交外部接口前，添加BID
            if (keys.filter(key => key.toLowerCase() === 'bid').length === 0) {
              res.BID = this.props.BID
            }
            
            resolve(res)
          } else {
            this.execError(res, btn)
            resolve(false)
            _resolve()
          }
        })
      } else {
        resolve(param)
      }
    }).then(res => {
      if (!res) return
      // 外部请求
      _outParam = JSON.parse(JSON.stringify(res))

      if (btn.outerFunc === 's_app_version_upt') { // 更新版本号
        if (window.GLOB.mainSystemApi) {
          res.rduri = window.GLOB.mainSystemApi
        }
      } else if (btn.sysInterface === 'true' && options.cloudServiceApi) {
        res.rduri = options.cloudServiceApi
      } else if (btn.sysInterface !== 'true') {
        res.rduri = btn.interface
      }

      if (btn.outerFunc) {
        res.func = btn.outerFunc
      }

      return Api.genericInterface(res)
    }).then(response => {
      if (!response) return
      // 回调请求
      if (btn.callbackFunc) {
        // 存在回调函数时，调用
        delete response.message
        delete response.status

        response.func = btn.callbackFunc

        if (response.UpType === 'SSO' && window.GLOB.mainSystemApi) {
          _localParam = {..._outParam, ...response}

          response.rduri = window.GLOB.mainSystemApi

          if (_localParam.LTextOut) {
            _localParam.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
            _localParam.secretkey = Utils.encrypt(_localParam.LTextOut, _localParam.timestamp)
            _localParam.open_key = Utils.encryptOpenKey(_localParam.secretkey, _localParam.timestamp)
          }
        }
        delete response.UpType

        let _callbackparam = {..._outParam, ...response}
        if (_callbackparam.LTextOut) {
          _callbackparam.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
          _callbackparam.secretkey = Utils.encrypt(_callbackparam.LTextOut, _callbackparam.timestamp)
          _callbackparam.open_key = Utils.encryptOpenKey(_callbackparam.secretkey, _callbackparam.timestamp)
        }

        return Api.getLocalConfig(_callbackparam)
      } else {
        if (response.status) {
          // 一次请求成功，进行下一项请求

          if (params.length === 0) {
            this.execSuccess(btn, response)
            _resolve()
          } else {
            this.outerLoopRequest(params, btn, _resolve)
          }
        } else {
          this.execError(response, btn)
          _resolve()
        }
      }
    }).then(res => {
      if (!res) return
      
      if (_localParam) {
        if (!res.status) {
          errRes = res
        }
        return Api.getLocalConfig(_localParam)
      } else if (res.status) {
        if (params.length === 0) {
          this.execSuccess(btn, res)
          _resolve()
        } else {
          this.outerLoopRequest(params, btn, _resolve)
        }
      } else {
        this.execError(res, btn)
        _resolve()
      }
    }).then(res => {
      if (!res) return

      if (res.status && !errRes) {
        if (params.length === 0) {
          this.execSuccess(btn, res)
          _resolve()
        } else {
          this.outerLoopRequest(params, btn, _resolve)
        }
      } else {
        this.execError(errRes || res, btn)
        _resolve()
      }
    })
  }

  /**
   * @description 操作成功后处理
   * 1、excel导出，成功后取消导出按钮加载中状态
   * 2、状态码为 S 时，显示成功信息后系统默认信息
   * 3、状态码为 -1 时，不显示任何信息
   * 4、模态框执行成功后是否关闭
   * 5、通知主列表刷新
   */
  execSuccess = (btn, res) => {
    if (res && res.ErrCode === 'S') { // 执行成功
      notification.success({
        top: 92,
        message: res.ErrMesg || this.props.dict['main.action.confirm.success'],
        duration: btn.verify && btn.verify.stime ? btn.verify.stime : 2
      })
    } else if (res && res.ErrCode === 'Y') { // 执行成功
      Modal.success({
        title: res.ErrMesg || this.props.dict['main.action.confirm.success']
      })
    } else if (res && res.ErrCode === '-1') { // 完成后不提示

    }
    
    if (btn.OpenType === 'pop' && btn.setting && btn.setting.finish !== 'unclose') {
      this.setState({
        visible: false
      })
    }

    this.setState({
      loadingUuid: '',
      loadingNumber: ''
    })

    this.refreshdata(btn, 'success')
  }

  /**
   * @description 操作失败后处理
   * 1、状态码为 E、N、F、NM 时，显示相应提示信息
   * 2、excel导出，失败后取消导出按钮加载中状态
   * 3、通知主列表刷新
   */
  execError = (res, btn) => {
    if (res.ErrCode === 'E') {
      Modal.error({
        title: res.message || res.ErrMesg,
      })
    } else if (res.ErrCode === 'N') {
      notification.error({
        top: 92,
        message: res.message || res.ErrMesg,
        duration: btn.verify && btn.verify.ntime ? btn.verify.ntime : 10
      })
    } else if (res.ErrCode === 'F') {
      notification.error({
        className: 'notification-custom-error',
        top: 92,
        message: res.message || res.ErrMesg,
        duration: btn.verify && btn.verify.ftime ? btn.verify.ftime : 10
      })
    } else if (res.ErrCode === 'NM') {
      message.error(res.message || res.ErrMesg)
    }
    
    this.setState({
      loadingUuid: '',
      loadingNumber: ''
    })
    
    this.refreshdata(btn, 'error')
  }

  modelconfirm = () => {
    const { BData } = this.props
    const { execAction, tabledata } = this.state
    let _this = this

    let result = []
    execAction.fields.forEach(item => {
      if (!item.field) return
      let _readin = item.readin !== 'false'
      let _initval = item.initval

      if (item.type === 'linkMain' || item.type === 'funcvar') {
        _readin = false
      }

      if (item.type === 'linkMain' && BData && BData.hasOwnProperty(item.field)) {
        _initval = BData[item.field]
      } else if (_readin && tabledata[0] && tabledata[0].hasOwnProperty(item.field)) {
        _initval = tabledata[0][item.field]
      }

      let _fieldlen = item.fieldlength || 50
      if (item.type === 'textarea' || item.type === 'fileupload' || item.type === 'multiselect') {
        _fieldlen = item.fieldlength || 512
      } else if (item.type === 'number') {
        _fieldlen = item.decimal ? item.decimal : 0
      }

      result.push({
        key: item.field,
        readonly: item.readonly === 'true',
        readin: _readin,
        fieldlen: _fieldlen,
        type: item.type,
        value: _initval
      })
    })

    confirm({
      title: this.props.dict['main.action.confirm.tip'],
      onOk() {
        _this.setState({
          loadingUuid: execAction.uuid
        })

        return new Promise(resolve => {
          _this.execSubmit(execAction, tabledata, resolve, result)
        })
      },
      onCancel() {}
    })
  }

  /**
   * @description 模态框（表单），确认
   */
  handleOk = () => {
    this.formRef.handleConfirm().then(res => {
      this.setState({
        confirmLoading: true
      })

      this.execSubmit(this.state.execAction, this.state.tabledata, () => {
        this.setState({
          confirmLoading: false
        })
      }, res)
    }, () => {})
  }

  /**
   * @description 模态框（表单），取消
   */
  handleCancel = () => {
    this.setState({
      visible: false
    })
  }

  /**
   * @description 显示模态框
   */
  getModels = () => {
    const { execAction } = this.state

    if (!execAction || !this.state.visible) return

    let title = ''
    let width = '62vw'
    let clickouter = false
    let container = document.body

    if (execAction && execAction.setting) {
      title = execAction.setting.title
      width = execAction.setting.width + 'vw'

      if (execAction.setting.container === 'tab' && this.props.ContainerId) {
        width = execAction.setting.width + '%'
        container = () => document.getElementById(this.props.ContainerId)
      }

      if (execAction.setting.clickouter === 'close') {
        clickouter = true
      }
    }

    if (this.props.type === 'subtab') {
      container = document.body
    }

    return (
      <Modal
        title={title}
        maskClosable={clickouter}
        getContainer={container}
        wrapClassName='action-modal'
        visible={this.state.visible}
        width={width}
        onOk={this.handleOk}
        confirmLoading={this.state.confirmLoading}
        onCancel={this.handleCancel}
        destroyOnClose
      >
        <MutilForm
          dict={this.props.dict}
          action={execAction}
          inputSubmit={this.handleOk}
          configMap={this.state.configMap}
          data={this.state.tabledata[0]}
          BData={this.props.BData}
          wrappedComponentRef={(inst) => this.formRef = inst}
        />
      </Modal>
    )
  }

  render() {
    const { loadingUuid, loadingNumber } = this.state

    return (
      <div className="verup-button-list verup-toolbar-button">
        {this.props.actions.map((item, index) => {
          let label = item.label
          if (loadingUuid === item.uuid && loadingNumber) {
            label = label + '(' + loadingNumber + ')'
          }
          return (
            <Button
              className={'mk-btn mk-' + item.class}
              icon={item.icon}
              key={'action' + index}
              onClick={() => {this.actionTrigger(item)}}
              loading={loadingUuid === item.uuid}
            >{label}</Button>
          )
        })}
        {this.getModels()}
      </div>
    )
  }
}

export default MainAction